<?php
declare(strict_types=1);

namespace Inti\Wp;

use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;
use Laminas\ConfigAggregator\PhpFileProvider;
use Laminas\ServiceManager\ServiceManager;
use Laminas\Stratigility\Middleware\ErrorHandler;
use Mezzio\Application;
use Mezzio\ConfigProvider as MezzioConfigProvider;
use Mezzio\Container;
use Mezzio\Container\ApplicationConfigInjectionDelegator;
use Mezzio\Handler\NotFoundHandler;
use Mezzio\Helper\ConfigProvider as MezzioHelperConfigProvider;
use Mezzio\Helper\ServerUrlMiddleware;
use Mezzio\Helper\UrlHelperMiddleware;
use Mezzio\Middleware\ErrorResponseGenerator;
use Mezzio\Router\ConfigProvider as MezzioRouterConfigProvider;
use Mezzio\Router\FastRouteRouter\ConfigProvider as MezzioRouterFastRouteConfigProvider;
use Mezzio\Router\Middleware\DispatchMiddleware;
use Mezzio\Router\Middleware\ImplicitHeadMiddleware;
use Mezzio\Router\Middleware\ImplicitOptionsMiddleware;
use Mezzio\Router\Middleware\MethodNotAllowedMiddleware;
use Mezzio\Router\Middleware\RouteMiddleware;
use function defined;

chdir(dirname(__DIR__));
require dirname(__DIR__) . '/vendor/autoload.php';

$config = (new ConfigAggregator([
    MezzioConfigProvider::class,
    MezzioHelperConfigProvider::class,
    MezzioRouterConfigProvider::class,
    MezzioRouterFastRouteConfigProvider::class,

    \Inti\Db\ConfigProvider::class,
    \Inti\Wp\Middleware\ConfigProvider::class,
    \Inti\Wp\Plugin\ConfigProvider::class,

    ConfigProvider::class,
    new PhpFileProvider(__DIR__ . '/config.local.php'),
    new ArrayProvider([
        'dependencies' => [
            'factories' => [
                ErrorResponseGenerator::class => Container\WhoopsErrorResponseGeneratorFactory::class,
                'Mezzio\Whoops' => Container\WhoopsFactory::class,
                'Mezzio\WhoopsPageHandler' => Container\WhoopsPageHandlerFactory::class,
            ],

            'delegators' => [
                Application::class => [
                    ApplicationConfigInjectionDelegator::class,
                ],
            ],
        ],

        'middleware_pipeline' => [[
            'middleware' => ErrorHandler::class,
            'priority' => 100,
        ], [
            'middleware' => ServerUrlMiddleware::class,
            'priority' => 100,
        ], [
            'middleware' => RouteMiddleware::class,
            'priority' => 75,
        ], [
            'middleware' => ImplicitHeadMiddleware::class,
            'priority' => 75,
        ], [
            'middleware' => ImplicitOptionsMiddleware::class,
            'priority' => 75,
        ], [
            'middleware' => MethodNotAllowedMiddleware::class,
            'priority' => 75,
        ], [
            'middleware' => UrlHelperMiddleware::class,
            'priority' => 75,
        ], [
            'middleware' => DispatchMiddleware::class,
            'priority' => 25,
        ], [
            'middleware' => NotFoundHandler::class,
            'priority' => -25,
        ]],

        'whoops' => [
            'json_exceptions' => [
                'display' => true,
                'show_trace' => true,
                'ajax_only' => true,
            ],
        ],
    ])
]))->getMergedConfig();

$container = new ServiceManager($config['dependencies']);
$container->setService('config', $config);

/** @var Application $app */
$app = $container->get(Application::class);
$app->run();

if (defined('WP_USE_THEMES')) {
    exit;
}
